# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-14 11:22
from __future__ import unicode_literals

from django.db import migrations, models
import tagging.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('when', models.DateTimeField()),
                ('what', models.CharField(max_length=255)),
                ('data', models.TextField(blank=True)),
                ('tags', tagging.fields.TagField(blank=True, default=b'', max_length=255)),
            ],
        ),
    ]
