{% load static %}<html>
  <head>
    <title>{{event.what}}</title>
    <link rel="stylesheet" type="text/css" href="{% static "css/table.css" %}" />
    <style type="text/css">
    body {
        font-family: sans-serif;
        font-size: 16px;
        margin: 50px;
        max-width: 1200px;
    }
    </style>


    </head>
    <body>
        <div id="title" style="text-align:center">
            <h1>{{event.what}}</h1>
        </div>
        <div class="graphite">
            <div id="main" >
              <table class="styledtable" width=100%>
                <tr><td>when</td><td>{{event.when|date:"H:i:s D d M Y" }}</td></tr>
                <tr><td>tags</td><td>{{event.tags}}</td></tr>
                <tr><td>data</td><td>{{event.data}}</td></tr>
              </table>
            </div>
        </div>
    </body>
</html>
