<!-- Copyright 2008 Orbitz WorldWide

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License. -->
<body onload="document.forms[0].username.focus();">

<center>

{% if authenticationFailed %}
<div style='color: red'>
<h3>Authentication attempt failed, please make sure you entered your login and password correctly<br>
</h3>
</div>
{% endif %}

{% if accountDisabled %}
<div class='color: red'>
<h3>Your account is <b>disabled</b>, please contact your site administrator.</h3>
</div>
{% endif %}

<form method='POST' action="{% url "account_login" %}">
{% if nextPage %}
<input type='hidden' name='nextPage' value='{{nextPage}}'/>
{% endif %}
 <table>
  <tr> <th colspan='2'>login</th> </tr>
  <tr> <td align='left'>username</td> <td align='left'><input type='text' name='username' class='textInput'></td> </tr>
  <tr> <td align='left'>password</td> <td align='left'><input type='password' name='password' class='textInput'></td> </tr>
  <tr> <td colspan='2' align='center'> <input type='submit' class='buttonInput' value='login'> </td> </tr>
 </table>
</form>
</center>
