{% load static %}<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
<!-- Copyright 2008 Orbitz WorldWide

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

   http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License. -->
<html>
  <head>
    <meta http-equiv="content-type" content="text/html; charset=ISO-8859-1">
    <title>Graphite Browser Header</title>
    <style>
      html {
        height: 100%;
      }
      body {
        background-color: #000;
        background: #000 url('{% static "img/carbon-fiber.png" %}');
        background: url('{% static "img/carbon-fiber.png" %}'), -webkit-linear-gradient(top, #000 55%, #333);
        background: url('{% static "img/carbon-fiber.png" %}'),    -moz-linear-gradient(top, #000, #000);
        background: url('{% static "img/carbon-fiber.png" %}'),     -ms-linear-gradient(top, #000 55%, #333);
        background: url('{% static "img/carbon-fiber.png" %}'),      -o-linear-gradient(top, #000 55%, #333);
        background: url('{% static "img/carbon-fiber.png" %}'),         linear-gradient(top, #000 55%, #333);
      }
      div.header {
        background-image: url('{% static "img/graphite-logo.png" %}');
        background-repeat: no-repeat;
        background-size: 12%;
        min-height: 60px;
        margin: 22px 0 0 50px;
      }
      .header div.nav {
        float: right;
      }
      .header .nav * {
        font: 0.96em Helvetica, Arial, Tahoma, sans-serif;
        color: rgb(173, 173, 173);
      }
      .header .nav ul {
        margin: 14px 30px 0 0;
        list-style: none;
      }
      .header .nav ul li {
        float: left;
        margin-right: 15px;
      }
      .header .nav ul li a {
        text-decoration: none;
      }
      .header .nav ul li.login a {
        color: #66B3CF;
      }
      .header .nav ul li a:hover, .header .nav ul li a.logout:hover {
        color: #fff;
      }
      .header .nav ul li a.logout {
        color: rgb(228, 133, 126);
      }
      .header .nav ul li.user_state {
        margin-left: 20px;
      }
    </style>
  </head>
  <body>
    <div class="header">
      <div class="nav">
        <ul>
        {% if documentation_url %}
          <li><a href="{{documentation_url}}" target="_newWindow">Documentation</a></li>
        {% endif %}
        <li><a href="{% url "dashboard" %}" target="_top">Dashboard</a></li>
        <li><a href="{% url "events" %}" target="_top">Events</a></li>
        {% if user.is_authenticated %}
          <li class="user_state"><a href="{% url "account_logout" %}" class="logout" target="_top">Logout</a> <b>{{ user.username }}</b> (<a href="{% url "account_edit" %}" target="_top">profile</a>)</li>
        {% else %}
          <li class="user_state login"><a href="{{login_url}}" target="_top">Login</a></li>
        {% endif %}
        </ul>
      </div>
    </div>
  </body>
</html>
