<style type='text/css'>
th {
  border-bottom: 1px solid black;
}
</style>
<center>
<table id='helpTable' width='90%' style='padding-top: 15px;'>
  <tr> <th width=120> <b>Shortcut</b> </th> <th> <b>Action</b> </th> </tr>
  <tr> <td>Ctrl-z</td> <td>Toggle visibility of the graph area toolbar</td> </tr>
  <tr> <td>Ctrl-space</td> <td>Toggle visibility of the navigation panel</td> </tr>
  <tr> <td>Alt-x</td> <td>Remove all graphs from the graph area</td> </tr>
  <tr> <td>Alt-s</td> <td>Save the current dashboard</td> </tr>
  <tr> <td></td> <td></td> </tr>
  <tr> <td></td> <td></td> </tr>
  <tr> <td></td> <td></td> </tr>
  <tr> <td></td> <td></td> </tr>
  <tr> <td></td> <td></td> </tr>
  <tr> <td></td> <td></td> </tr>
  <tr>
    <td colspan='2' style='text-align: center; color: rgb(150, 150, 150);'>
      <i>The following shortcuts are for the Completer UI mode only</i>
    </td>
  </tr>
  <tr> <td>Alt-Enter</td> <td>Add all matching metrics to the graph area</td> </tr>
  <tr> <td>Alt-Backspace</td> <td>Remove all matching metrics from the graph area</td> </tr>
  <tr> <td>Shift-space</td> <td>Put keyboard focus in the completer field</td> </tr>
</table>
</center>
